// Copyright 2021 99cloud
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import { inject, observer } from 'mobx-react';
import { ModalAction } from 'containers/Action';
import globalFloatingIpsStore from 'stores/neutron/floatingIp';
import { getQoSPolicyTabs } from 'resources/neutron/qos-policy';
import { QoSPolicyStore } from 'stores/neutron/qos-policy';
import { qosEndpoint } from 'client/client/constants';
import React from 'react';

export class Edit extends ModalAction {
  static id = 'edit-floating-ip';

  static policy = 'update_floatingip';

  static get modalSize() {
    return qosEndpoint() ? 'large' : 'small';
  }

  getModalSize() {
    return qosEndpoint() ? 'large' : 'large';
  }

  get qosEndpoint() {
    return qosEndpoint();
  }

  init() {
    this.qosPolicyStore = new QoSPolicyStore();
  }

  get defaultValue() {
    const { item } = this.props;
    let ptrRecord = '';
    try {
      const floatingIps = JSON.parse(localStorage.getItem('floatingIps'));
      const matchingIp = floatingIps.find(
        (ip) => ip.floating_ip_address === item.floating_ip_address
      );
      if (matchingIp) {
        ptrRecord = matchingIp.ptr_record;
      }
    } catch (error) {
      console.log(error);
    }
    return {
      description: this.item.description,
      reversedns: ptrRecord,
      dnsptr: this.item.dnsptr ? this.item.dnsptr : '',
      ...(this.qosEndpoint
        ? {
            qos_policy_id: {
              selectedRowKeys: item.qos_policy_id ? [item.qos_policy_id] : [],
              selectedRows: item.qos_policy_id
                ? [
                    {
                      id: item.qos_policy_id,
                      name: item.qos_policy_id,
                    },
                  ]
                : [],
            },
          }
        : {}),
    };
  }

  get instanceName() {
    return this.item.floating_ip_address;
  }

  static allowed = () => Promise.resolve(true);

  onSubmit = (values) => {
    const { description, qos_policy_id, reversedns } = values;
    const body = {
      description,
    };
    if (this.qosEndpoint) {
      body.qos_policy_id = qos_policy_id
        ? qos_policy_id.selectedRowKeys.length
          ? qos_policy_id.selectedRowKeys[0]
          : null
        : null;
    }
    try {
      const { floating_ip_address } = this.item;
      const userEmail = localStorage.getItem('userEmail') || '';
      const path = localStorage.getItem('consoleHost');
      fetch(`${path}/api/reversedns`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          reversedns,
          floating_ip_address,
          user_email: userEmail,
        }),
      })
        .then((response) => response.json())
        .then((data) => {
          if (data.status === 'success') {
            const floatingIps =
              JSON.parse(localStorage.getItem('floatingIps')) || [];
            const updatedIps = floatingIps.map((ip) =>
              ip.floating_ip_address === floating_ip_address
                ? { ...ip, ptr_record: reversedns }
                : ip
            );
            localStorage.setItem('floatingIps', JSON.stringify(updatedIps));
          }
        })
        .catch((error) => {
          console.error('Error sending to Flask app:', error);
        });
    } catch (error) {
      console.log(error);
    }

    return globalFloatingIpsStore.edit({ id: this.item.id }, body);
  };

  get formItems() {
    return [
      {
        name: 'description',
        label: t('Description'),
        type: 'textarea',
      },
      {
        name: 'qos_policy_id',
        label: t('QoS Policy'),
        type: 'tab-select-table',
        tabs: getQoSPolicyTabs.call(this),
        isMulti: false,
        tip: t('Choosing a QoS policy can limit bandwidth and DSCP'),
        display: !!this.qosEndpoint,
      },
      {
        name: 'reversedns',
        label: 'Reverse DNS',
        component: (
          <input
            type="text"
            placeholder="Reverse DNS"
            name="reversedns"
            className="ant-input"
          />
        ),
      },
    ];
  }
}

export default inject('rootStore')(observer(Edit));
